@@ -145,7 +145,7 @@ void CameraMatrix::set_for_hmd(int p_eye, real_t p_aspect, real_t p_intraocular_
 	f3 *= p_oversample;
 
 	// always apply KEEP_WIDTH aspect ratio
-	f3 *= p_aspect;
+	f3 /= p_aspect;
 
 	switch (p_eye) {
 		case 1: { // left eye
