@@ -116,27 +116,25 @@ void FindReplaceBar::_notification(int p_what) {
 void FindReplaceBar::_unhandled_input(const Ref<InputEvent> &p_event) {
 
 	Ref<InputEventKey> k = p_event;
-	if (k.is_valid()) {
+	if (!k.is_valid() || !k->is_pressed()) {
+		return;
+	}
 
-		if (k->is_pressed() && (text_edit->has_focus() || vbc_lineedit->is_a_parent_of(get_focus_owner()))) {
+	Control *focus_owner = get_focus_owner();
+	if (text_edit->has_focus() || (focus_owner && vbc_lineedit->is_a_parent_of(focus_owner))) {
+		bool accepted = true;
 
-			bool accepted = true;
+		switch (k->get_scancode()) {
+			case KEY_ESCAPE: {
+				_hide_bar();
+			} break;
+			default: {
+				accepted = false;
+			} break;
+		}
 
-			switch (k->get_scancode()) {
-
-				case KEY_ESCAPE: {
-
-					_hide_bar();
-				} break;
-				default: {
-
-					accepted = false;
-				} break;
-			}
-
-			if (accepted) {
-				accept_event();
-			}
+		if (accepted) {
+			accept_event();
 		}
 	}
 }
@@ -180,21 +178,47 @@ bool FindReplaceBar::_search(uint32_t p_flags, int p_from_line, int p_from_col)
 }
 
 void FindReplaceBar::_replace() {
-
-	if (result_line != -1 && result_col != -1) {
-		text_edit->begin_complex_operation();
-
-		text_edit->unfold_line(result_line);
-		text_edit->select(result_line, result_col, result_line, result_col + get_search_text().length());
-		text_edit->insert_text_at_cursor(get_replace_text());
-
-		text_edit->end_complex_operation();
-
-		results_count = -1;
+	bool selection_enabled = text_edit->is_selection_active();
+	Point2i selection_begin, selection_end;
+	if (selection_enabled) {
+		selection_begin = Point2i(text_edit->get_selection_from_line(), text_edit->get_selection_from_column());
+		selection_end = Point2i(text_edit->get_selection_to_line(), text_edit->get_selection_to_column());
 	}
 
-	if (!search_current()) {
-		search_next();
+	String replace_text = get_replace_text();
+	int search_text_len = get_search_text().length();
+
+	text_edit->begin_complex_operation();
+	if (selection_enabled && is_selection_only()) { // To restrict search_current() to selected region
+		text_edit->cursor_set_line(selection_begin.width);
+		text_edit->cursor_set_column(selection_begin.height);
+	}
+
+	if (search_current()) {
+		text_edit->unfold_line(result_line);
+		text_edit->select(result_line, result_col, result_line, result_col + search_text_len);
+
+		if (selection_enabled && is_selection_only()) {
+			Point2i match_from(result_line, result_col);
+			Point2i match_to(result_line, result_col + search_text_len);
+			if (!(match_from < selection_begin || match_to > selection_end)) {
+				text_edit->insert_text_at_cursor(replace_text);
+				if (match_to.x == selection_end.x) { // Adjust selection bounds if necessary
+					selection_end.y += replace_text.length() - search_text_len;
+				}
+			}
+		} else {
+			text_edit->insert_text_at_cursor(replace_text);
+		}
+	}
+	text_edit->end_complex_operation();
+	results_count = -1;
+
+	if (selection_enabled && is_selection_only()) {
+		// Reselect in order to keep 'Replace' restricted to selection
+		text_edit->select(selection_begin.x, selection_begin.y, selection_end.x, selection_end.y);
+	} else {
+		text_edit->deselect();
 	}
 }
 
@@ -226,6 +250,10 @@ void FindReplaceBar::_replace_all() {
 
 	text_edit->begin_complex_operation();
 
+	if (selection_enabled && is_selection_only()) {
+		text_edit->cursor_set_line(selection_begin.width);
+		text_edit->cursor_set_column(selection_begin.height);
+	}
 	if (search_current()) {
 		do {
 			// replace area
@@ -243,7 +271,7 @@ void FindReplaceBar::_replace_all() {
 
 			if (selection_enabled && is_selection_only()) {
 				if (match_from < selection_begin || match_to > selection_end) {
-					continue;
+					break; // Done.
 				}
 
 				// Replace but adjust selection bounds.
@@ -277,7 +305,8 @@ void FindReplaceBar::_replace_all() {
 	}
 
 	text_edit->set_v_scroll(vsval);
-	set_error(vformat(TTR("Replaced %d occurrence(s)."), rc));
+	matches_label->add_color_override("font_color", rc > 0 ? get_color("font_color", "Label") : get_color("error_color", "Editor"));
+	matches_label->set_text(vformat(TTR("%d replaced."), rc));
 
 	text_edit->call_deferred("connect", "text_changed", this, "_editor_text_changed");
 	results_count = -1;
@@ -288,6 +317,10 @@ void FindReplaceBar::_get_search_from(int &r_line, int &r_col) {
 	r_line = text_edit->cursor_get_line();
 	r_col = text_edit->cursor_get_column();
 
+	if (text_edit->is_selection_active() && is_selection_only()) {
+		return;
+	}
+
 	if (r_line == result_line && r_col >= result_col && r_col <= result_col + get_search_text().length()) {
 		r_col = result_col;
 	}
@@ -310,16 +343,17 @@ void FindReplaceBar::_update_results_count() {
 		int pos = is_case_sensitive() ? full_text.find(searched, from_pos) : full_text.findn(searched, from_pos);
 		if (pos == -1) break;
 
+		int pos_subsequent = pos + searched.length();
 		if (is_whole_words()) {
-			from_pos++; // Making sure we won't hit the same match next time, if we get out via a continue.
-			if (pos > 0 && !is_symbol(full_text[pos - 1]))
+			from_pos = pos + 1; // Making sure we won't hit the same match next time, if we get out via a continue.
+			if (pos > 0 && !(is_symbol(full_text[pos - 1]) || full_text[pos - 1] == '\n'))
 				continue;
-			if (pos + searched.length() < full_text.length() && !is_symbol(full_text[pos + searched.length()]))
+			if (pos_subsequent < full_text.length() && !(is_symbol(full_text[pos_subsequent]) || full_text[pos_subsequent] == '\n'))
 				continue;
 		}
 
 		results_count++;
-		from_pos = pos + searched.length();
+		from_pos = pos_subsequent;
 	}
 }
 
@@ -685,6 +719,16 @@ void CodeTextEditor::_input(const Ref<InputEvent> &event) {
 		accept_event();
 		return;
 	}
+	if (ED_IS_SHORTCUT("script_text_editor/delete_line", key_event)) {
+		delete_lines();
+		accept_event();
+		return;
+	}
+	if (ED_IS_SHORTCUT("script_text_editor/clone_down", key_event)) {
+		clone_lines_down();
+		accept_event();
+		return;
+	}
 }
 
 void CodeTextEditor::_text_editor_gui_input(const Ref<InputEvent> &p_event) {
@@ -1250,7 +1294,7 @@ void CodeTextEditor::clone_lines_down() {
 	text_editor->cursor_set_line(cursor_new_line);
 	text_editor->cursor_set_column(cursor_new_column);
 	if (selection_active) {
-		text_editor->select(to_line, to_column, 2 * to_line - from_line, 2 * to_column - from_column);
+		text_editor->select(to_line, to_column, 2 * to_line - from_line, to_line == from_line ? 2 * to_column - from_column : to_column);
 	}
 
 	text_editor->end_complex_operation();
@@ -1515,7 +1559,7 @@ void CodeTextEditor::_toggle_scripts_pressed() {
 void CodeTextEditor::_error_pressed(const Ref<InputEvent> &p_event) {
 	Ref<InputEventMouseButton> mb = p_event;
 	if (mb.is_valid() && mb->is_pressed() && mb->get_button_index() == BUTTON_LEFT) {
-		emit_signal("error_pressed");
+		goto_error();
 	}
 }
 
@@ -1642,7 +1686,6 @@ void CodeTextEditor::_bind_methods() {
 	ADD_SIGNAL(MethodInfo("validate_script"));
 	ADD_SIGNAL(MethodInfo("load_theme_settings"));
 	ADD_SIGNAL(MethodInfo("show_warnings_panel"));
-	ADD_SIGNAL(MethodInfo("error_pressed"));
 }
 
 void CodeTextEditor::set_code_complete_func(CodeTextEditorCodeCompleteFunc p_code_complete_func, void *p_ud) {
