@@ -35,7 +35,10 @@
 #include "editor_icons.gen.h"
 #include "editor_scale.h"
 #include "editor_settings.h"
+
+#ifdef SVG_ENABLED
 #include "modules/svg/image_loader_svg.h"
+#endif
 
 static Ref<StyleBoxTexture> make_stylebox(Ref<Texture> p_texture, float p_left, float p_top, float p_right, float p_botton, float p_margin_left = -1, float p_margin_top = -1, float p_margin_right = -1, float p_margin_botton = -1, bool p_draw_center = true) {
 	Ref<StyleBoxTexture> style(memnew(StyleBoxTexture));
@@ -81,7 +84,8 @@ static Ref<StyleBoxLine> make_line_stylebox(Color p_color, int p_thickness = 1,
 	return style;
 }
 
-Ref<ImageTexture> editor_generate_icon(int p_index, bool p_convert_color, float p_scale = EDSCALE, bool p_force_filter = false) {
+#ifdef SVG_ENABLED
+static Ref<ImageTexture> editor_generate_icon(int p_index, bool p_convert_color, float p_scale = EDSCALE, bool p_force_filter = false) {
 
 	Ref<ImageTexture> icon = memnew(ImageTexture);
 	Ref<Image> img = memnew(Image);
@@ -89,7 +93,11 @@ Ref<ImageTexture> editor_generate_icon(int p_index, bool p_convert_color, float
 	// dumb gizmo check
 	bool is_gizmo = String(editor_icons_names[p_index]).begins_with("Gizmo");
 
-	ImageLoaderSVG::create_image_from_string(img, editor_icons_sources[p_index], p_scale, true, p_convert_color);
+	// Upsample icon generation only if the editor scale isn't an integer multiplier.
+	// Generating upsampled icons is slower, and the benefit is hardly visible
+	// with integer editor scales.
+	const bool upsample = !Math::is_equal_approx(Math::round(p_scale), p_scale);
+	ImageLoaderSVG::create_image_from_string(img, editor_icons_sources[p_index], p_scale, upsample, p_convert_color);
 
 	if ((p_scale - (float)((int)p_scale)) > 0.0 || is_gizmo || p_force_filter)
 		icon->create_from_image(img); // in this case filter really helps
@@ -98,6 +106,7 @@ Ref<ImageTexture> editor_generate_icon(int p_index, bool p_convert_color, float
 
 	return icon;
 }
+#endif
 
 #ifndef ADD_CONVERT_COLOR
 #define ADD_CONVERT_COLOR(dictionary, old_color, new_color) dictionary[Color::html(old_color)] = Color::html(new_color)
@@ -106,7 +115,15 @@ Ref<ImageTexture> editor_generate_icon(int p_index, bool p_convert_color, float
 void editor_register_and_generate_icons(Ref<Theme> p_theme, bool p_dark_theme = true, int p_thumb_size = 32, bool p_only_thumbs = false) {
 
 #ifdef SVG_ENABLED
+	// The default icon theme is designed to be used for a dark theme.
+	// This dictionary stores color codes to convert to other colors
+	// for better readability on a light theme.
 	Dictionary dark_icon_color_dictionary;
+
+	// The names of the icons to never convert, even if one of their colors
+	// are contained in the dictionary above.
+	Set<StringName> exceptions;
+
 	if (!p_dark_theme) {
 		// convert color:                             FROM       TO
 		ADD_CONVERT_COLOR(dark_icon_color_dictionary, "#e0e0e0", "#5a5a5a"); // common icon color
@@ -172,9 +189,31 @@ void editor_register_and_generate_icons(Ref<Theme> p_theme, bool p_dark_theme =
 		ADD_CONVERT_COLOR(dark_icon_color_dictionary, "#69ec9a", "#2ce573"); // VS rid
 		ADD_CONVERT_COLOR(dark_icon_color_dictionary, "#79f3e8", "#12d5c3"); // VS object
 		ADD_CONVERT_COLOR(dark_icon_color_dictionary, "#77edb1", "#57e99f"); // VS dict
+
+		exceptions.insert("EditorPivot");
+		exceptions.insert("EditorHandle");
+		exceptions.insert("Editor3DHandle");
+		exceptions.insert("Godot");
+		exceptions.insert("PanoramaSky");
+		exceptions.insert("ProceduralSky");
+		exceptions.insert("EditorControlAnchor");
+		exceptions.insert("DefaultProjectIcon");
+		exceptions.insert("GuiCloseCustomizable");
+		exceptions.insert("GuiGraphNodePort");
+		exceptions.insert("GuiResizer");
+		exceptions.insert("ZoomMore");
+		exceptions.insert("ZoomLess");
+		exceptions.insert("ZoomReset");
+		exceptions.insert("LockViewport");
+		exceptions.insert("GroupViewport");
+		exceptions.insert("StatusError");
+		exceptions.insert("StatusSuccess");
+		exceptions.insert("StatusWarning");
+		exceptions.insert("NodeWarning");
+		exceptions.insert("OverbrightIndicator");
 	}
 
-	// these ones should be converted even if we are using a dark theme
+	// These ones should be converted even if we are using a dark theme.
 	const Color error_color = p_theme->get_color("error_color", "Editor");
 	const Color success_color = p_theme->get_color("success_color", "Editor");
 	const Color warning_color = p_theme->get_color("warning_color", "Editor");
@@ -182,65 +221,44 @@ void editor_register_and_generate_icons(Ref<Theme> p_theme, bool p_dark_theme =
 	dark_icon_color_dictionary[Color::html("#45ff8b")] = success_color;
 	dark_icon_color_dictionary[Color::html("#dbab09")] = warning_color;
 
-	List<String> exceptions;
-	exceptions.push_back("EditorPivot");
-	exceptions.push_back("EditorHandle");
-	exceptions.push_back("Editor3DHandle");
-	exceptions.push_back("Godot");
-	exceptions.push_back("PanoramaSky");
-	exceptions.push_back("ProceduralSky");
-	exceptions.push_back("EditorControlAnchor");
-	exceptions.push_back("DefaultProjectIcon");
-	exceptions.push_back("GuiCloseCustomizable");
-	exceptions.push_back("GuiGraphNodePort");
-	exceptions.push_back("GuiResizer");
-	exceptions.push_back("ZoomMore");
-	exceptions.push_back("ZoomLess");
-	exceptions.push_back("ZoomReset");
-	exceptions.push_back("LockViewport");
-	exceptions.push_back("GroupViewport");
-	exceptions.push_back("StatusError");
-	exceptions.push_back("StatusSuccess");
-	exceptions.push_back("StatusWarning");
-	exceptions.push_back("NodeWarning");
-	exceptions.push_back("OverbrightIndicator");
-
 	ImageLoaderSVG::set_convert_colors(&dark_icon_color_dictionary);
 
-	// generate icons
-	if (!p_only_thumbs)
+	// Generate icons.
+	if (!p_only_thumbs) {
 		for (int i = 0; i < editor_icons_count; i++) {
-			List<String>::Element *is_exception = exceptions.find(editor_icons_names[i]);
-			if (is_exception) exceptions.erase(is_exception);
-			Ref<ImageTexture> icon = editor_generate_icon(i, !is_exception);
+			const int is_exception = exceptions.has(editor_icons_names[i]);
+			const Ref<ImageTexture> icon = editor_generate_icon(i, !is_exception);
+
 			p_theme->set_icon(editor_icons_names[i], "EditorIcons", icon);
 		}
+	}
 
-	// generate thumb files with the given thumb size
-	bool force_filter = p_thumb_size != 64 && p_thumb_size != 32; // we don't need filter with original resolution
+	// Generate thumbnail icons with the given thumbnail size.
+	// We don't need filtering when generating at one of the default resolutions.
+	const bool force_filter = p_thumb_size != 64 && p_thumb_size != 32;
 	if (p_thumb_size >= 64) {
-		float scale = (float)p_thumb_size / 64.0 * EDSCALE;
+		const float scale = (float)p_thumb_size / 64.0 * EDSCALE;
 		for (int i = 0; i < editor_bg_thumbs_count; i++) {
-			int index = editor_bg_thumbs_indices[i];
-			List<String>::Element *is_exception = exceptions.find(editor_icons_names[index]);
-			if (is_exception) exceptions.erase(is_exception);
-			Ref<ImageTexture> icon = editor_generate_icon(index, !p_dark_theme && !is_exception, scale, force_filter);
+			const int index = editor_bg_thumbs_indices[i];
+			const int is_exception = exceptions.has(editor_icons_names[index]);
+			const Ref<ImageTexture> icon = editor_generate_icon(index, !p_dark_theme && !is_exception, scale, force_filter);
+
 			p_theme->set_icon(editor_icons_names[index], "EditorIcons", icon);
 		}
 	} else {
-		float scale = (float)p_thumb_size / 32.0 * EDSCALE;
+		const float scale = (float)p_thumb_size / 32.0 * EDSCALE;
 		for (int i = 0; i < editor_md_thumbs_count; i++) {
-			int index = editor_md_thumbs_indices[i];
-			List<String>::Element *is_exception = exceptions.find(editor_icons_names[index]);
-			if (is_exception) exceptions.erase(is_exception);
-			Ref<ImageTexture> icon = editor_generate_icon(index, !p_dark_theme && !is_exception, scale, force_filter);
+			const int index = editor_md_thumbs_indices[i];
+			const bool is_exception = exceptions.has(editor_icons_names[index]);
+			const Ref<ImageTexture> icon = editor_generate_icon(index, !p_dark_theme && !is_exception, scale, force_filter);
+
 			p_theme->set_icon(editor_icons_names[index], "EditorIcons", icon);
 		}
 	}
 
 	ImageLoaderSVG::set_convert_colors(NULL);
 #else
-	print_line("SVG support disabled, editor icons won't be rendered.");
+	WARN_PRINT("SVG support disabled, editor icons won't be rendered.");
 #endif
 }
 
@@ -842,6 +860,7 @@ Ref<Theme> create_editor_theme(const Ref<Theme> p_theme) {
 	Ref<StyleBoxFlat> style_panel_invisible_top = style_content_panel->duplicate();
 	int stylebox_offset = theme->get_font("tab_fg", "TabContainer")->get_height() + theme->get_stylebox("tab_fg", "TabContainer")->get_minimum_size().height + theme->get_stylebox("panel", "TabContainer")->get_default_margin(MARGIN_TOP);
 	style_panel_invisible_top->set_expand_margin_size(MARGIN_TOP, -stylebox_offset);
+	style_panel_invisible_top->set_default_margin(MARGIN_TOP, 0);
 	theme->set_stylebox("BottomPanelDebuggerOverride", "EditorStyles", style_panel_invisible_top);
 
 	// LineEdit
@@ -944,12 +963,14 @@ Ref<Theme> create_editor_theme(const Ref<Theme> p_theme) {
 	theme->set_icon("grabber", "HSlider", theme->get_icon("GuiSliderGrabber", "EditorIcons"));
 	theme->set_stylebox("slider", "HSlider", make_flat_stylebox(dark_color_3, 0, default_margin_size / 2, 0, default_margin_size / 2));
 	theme->set_stylebox("grabber_area", "HSlider", make_flat_stylebox(contrast_color_1, 0, default_margin_size / 2, 0, default_margin_size / 2));
+	theme->set_stylebox("grabber_area_highlight", "HSlider", make_flat_stylebox(contrast_color_1, 0, default_margin_size / 2, 0, default_margin_size / 2));
 
 	// VSlider
 	theme->set_icon("grabber", "VSlider", theme->get_icon("GuiSliderGrabber", "EditorIcons"));
 	theme->set_icon("grabber_highlight", "VSlider", theme->get_icon("GuiSliderGrabberHl", "EditorIcons"));
 	theme->set_stylebox("slider", "VSlider", make_flat_stylebox(dark_color_3, default_margin_size / 2, 0, default_margin_size / 2, 0));
 	theme->set_stylebox("grabber_area", "VSlider", make_flat_stylebox(contrast_color_1, default_margin_size / 2, 0, default_margin_size / 2, 0));
+	theme->set_stylebox("grabber_area_highlight", "VSlider", make_flat_stylebox(contrast_color_1, default_margin_size / 2, 0, default_margin_size / 2, 0));
 
 	//RichTextLabel
 	theme->set_color("default_color", "RichTextLabel", font_color);
@@ -977,6 +998,9 @@ Ref<Theme> create_editor_theme(const Ref<Theme> p_theme) {
 	// LinkButton
 	theme->set_stylebox("focus", "LinkButton", style_empty);
 	theme->set_color("font_color", "LinkButton", font_color);
+	theme->set_color("font_color_hover", "LinkButton", font_color_hl);
+	theme->set_color("font_color_pressed", "LinkButton", accent_color);
+	theme->set_color("font_color_disabled", "LinkButton", font_color_disabled);
 
 	// TooltipPanel
 	Ref<StyleBoxFlat> style_tooltip = style_popup->duplicate();
