@@ -76,7 +76,7 @@ void ConnectionInfoDialog::popup_connections(String p_method, Vector<Node *> p_n
 		}
 	}
 
-	popup_centered(Size2(400, 300) * EDSCALE);
+	popup_centered(Size2(600, 300) * EDSCALE);
 }
 
 ConnectionInfoDialog::ConnectionInfoDialog() {
@@ -380,10 +380,6 @@ void ScriptTextEditor::_show_warnings_panel(bool p_show) {
 	warnings_panel->set_visible(p_show);
 }
 
-void ScriptTextEditor::_error_pressed() {
-	code_editor->goto_error();
-}
-
 void ScriptTextEditor::_warning_clicked(Variant p_line) {
 	if (p_line.get_type() == Variant::INT) {
 		code_editor->get_text_edit()->cursor_set_line(p_line.operator int64_t());
@@ -625,6 +621,18 @@ void ScriptTextEditor::_validate_script() {
 	for (List<ScriptLanguage::Warning>::Element *E = warnings.front(); E; E = E->next()) {
 		ScriptLanguage::Warning w = E->get();
 
+		Dictionary ignore_meta;
+		ignore_meta["line"] = w.line;
+		ignore_meta["code"] = w.string_code.to_lower();
+		warnings_panel->push_cell();
+		warnings_panel->push_meta(ignore_meta);
+		warnings_panel->push_color(
+				warnings_panel->get_color("accent_color", "Editor").linear_interpolate(warnings_panel->get_color("mono_color", "Editor"), 0.5));
+		warnings_panel->add_text(TTR("[Ignore]"));
+		warnings_panel->pop(); // Color.
+		warnings_panel->pop(); // Meta ignore.
+		warnings_panel->pop(); // Cell.
+
 		warnings_panel->push_cell();
 		warnings_panel->push_meta(w.line - 1);
 		warnings_panel->push_color(warnings_panel->get_color("warning_color", "Editor"));
@@ -637,15 +645,6 @@ void ScriptTextEditor::_validate_script() {
 		warnings_panel->push_cell();
 		warnings_panel->add_text(w.message);
 		warnings_panel->pop(); // Cell.
-
-		Dictionary ignore_meta;
-		ignore_meta["line"] = w.line;
-		ignore_meta["code"] = w.string_code.to_lower();
-		warnings_panel->push_cell();
-		warnings_panel->push_meta(ignore_meta);
-		warnings_panel->add_text(TTR("(ignore)"));
-		warnings_panel->pop(); // Meta ignore.
-		warnings_panel->pop(); // Cell.
 	}
 	warnings_panel->pop(); // Table.
 
@@ -970,9 +969,35 @@ void ScriptTextEditor::_lookup_symbol(const String &p_symbol, int p_row, int p_c
 				emit_signal("go_to_help", "class_global:" + result.class_name + ":" + result.class_member);
 			} break;
 		}
+	} else if (ProjectSettings::get_singleton()->has_setting("autoload/" + p_symbol)) {
+		//check for Autoload scenes
+		String path = ProjectSettings::get_singleton()->get("autoload/" + p_symbol);
+		if (path.begins_with("*")) {
+			path = path.substr(1, path.length());
+			EditorNode::get_singleton()->load_scene(path);
+		}
+	} else if (p_symbol.is_rel_path()) {
+		// Every symbol other than absolute path is relative path so keep this condition at last.
+		String path = _get_absolute_path(p_symbol);
+		if (FileAccess::exists(path)) {
+			List<String> scene_extensions;
+			ResourceLoader::get_recognized_extensions_for_type("PackedScene", &scene_extensions);
+
+			if (scene_extensions.find(path.get_extension())) {
+				EditorNode::get_singleton()->load_scene(path);
+			} else {
+				EditorNode::get_singleton()->load_resource(path);
+			}
+		}
 	}
 }
 
+String ScriptTextEditor::_get_absolute_path(const String &rel_path) {
+	String base_path = script->get_path().get_base_dir();
+	String path = base_path.plus_file(rel_path);
+	return path.replace("///", "//").simplify_path();
+}
+
 void ScriptTextEditor::update_toggle_scripts_button() {
 	if (code_editor != NULL) {
 		code_editor->update_toggle_scripts_button();
@@ -1438,7 +1463,6 @@ void ScriptTextEditor::_bind_methods() {
 	ClassDB::bind_method("_lookup_symbol", &ScriptTextEditor::_lookup_symbol);
 	ClassDB::bind_method("_text_edit_gui_input", &ScriptTextEditor::_text_edit_gui_input);
 	ClassDB::bind_method("_show_warnings_panel", &ScriptTextEditor::_show_warnings_panel);
-	ClassDB::bind_method("_error_pressed", &ScriptTextEditor::_error_pressed);
 	ClassDB::bind_method("_warning_clicked", &ScriptTextEditor::_warning_clicked);
 	ClassDB::bind_method("_color_changed", &ScriptTextEditor::_color_changed);
 
@@ -1779,6 +1803,8 @@ ScriptTextEditor::ScriptTextEditor() {
 
 	warnings_panel = memnew(RichTextLabel);
 	editor_box->add_child(warnings_panel);
+	warnings_panel->add_font_override(
+			"normal_font", EditorNode::get_singleton()->get_gui_base()->get_font("main", "EditorFonts"));
 	warnings_panel->set_custom_minimum_size(Size2(0, 100 * EDSCALE));
 	warnings_panel->set_h_size_flags(SIZE_EXPAND_FILL);
 	warnings_panel->set_meta_underline(true);
@@ -1786,7 +1812,6 @@ ScriptTextEditor::ScriptTextEditor() {
 	warnings_panel->set_focus_mode(FOCUS_CLICK);
 	warnings_panel->hide();
 
-	code_editor->connect("error_pressed", this, "_error_pressed");
 	code_editor->connect("show_warnings_panel", this, "_show_warnings_panel");
 	warnings_panel->connect("meta_clicked", this, "_warning_clicked");
 
