@@ -928,6 +928,10 @@ Variant SceneTreeEditor::get_drag_data_fw(const Point2 &p_point, Control *p_from
 	if (!can_rename)
 		return Variant(); //not editable tree
 
+	if (tree->get_button_id_at_position(p_point) != -1) {
+		return Variant(); //dragging from button
+	}
+
 	Vector<Node *> selected;
 	Vector<Ref<Texture> > icons;
 	TreeItem *next = tree->get_next_selected(NULL);
