@@ -52,10 +52,9 @@ void ScriptCreateDialog::_notification(int p_what) {
 					language_menu->set_item_icon(i, lang_icon);
 				}
 			}
-			String last_lang = EditorSettings::get_singleton()->get_project_metadata("script_setup", "last_selected_language", "");
-			Ref<Texture> last_lang_icon;
-			if (!last_lang.empty()) {
 
+			String last_lang = EditorSettings::get_singleton()->get_project_metadata("script_setup", "last_selected_language", "");
+			if (!last_lang.empty()) {
 				for (int i = 0; i < language_menu->get_item_count(); i++) {
 					if (language_menu->get_item_text(i) == last_lang) {
 						language_menu->select(i);
@@ -63,14 +62,10 @@ void ScriptCreateDialog::_notification(int p_what) {
 						break;
 					}
 				}
-
-				last_lang_icon = get_icon(last_lang, "EditorIcons");
 			} else {
-				last_lang_icon = language_menu->get_item_icon(default_language);
-			}
-			if (last_lang_icon.is_valid()) {
-				language_menu->set_icon(last_lang_icon);
+				language_menu->select(default_language);
 			}
+
 			path_button->set_icon(get_icon("Folder", "EditorIcons"));
 			parent_browse_button->set_icon(get_icon("Folder", "EditorIcons"));
 			parent_search_button->set_icon(get_icon("ClassList", "EditorIcons"));
@@ -84,7 +79,9 @@ void ScriptCreateDialog::_path_hbox_sorted() {
 		int filename_start_pos = initial_bp.find_last("/") + 1;
 		int filename_end_pos = initial_bp.length();
 
-		file_path->select(filename_start_pos, filename_end_pos);
+		if (!is_built_in) {
+			file_path->select(filename_start_pos, filename_end_pos);
+		}
 
 		// First set cursor to the end of line to scroll LineEdit view
 		// to the right and then set the actual cursor position.
@@ -513,6 +510,7 @@ void ScriptCreateDialog::_built_in_pressed() {
 		_path_changed(file_path->get_text());
 	}
 	_update_dialog();
+	minimum_size_changed();
 }
 
 void ScriptCreateDialog::_browse_path(bool browse_parent, bool p_save) {
@@ -575,6 +573,10 @@ void ScriptCreateDialog::_browse_class_in_tree() {
 
 void ScriptCreateDialog::_path_changed(const String &p_path) {
 
+	if (is_built_in) {
+		return;
+	}
+
 	is_path_valid = false;
 	is_new_script_created = true;
 
@@ -644,7 +646,7 @@ void ScriptCreateDialog::_update_dialog() {
 	}
 
 	if (script_ok) {
-		_msg_script_valid(true, TTR("Script is valid."));
+		_msg_script_valid(true, TTR("Script path/name is valid."));
 	}
 
 	// Does script have named classes?
@@ -686,6 +688,8 @@ void ScriptCreateDialog::_update_dialog() {
 
 	// Is Script created or loaded from existing file?
 
+	builtin_warning_label->set_visible(is_built_in);
+
 	if (is_built_in) {
 		get_ok()->set_text(TTR("Create"));
 		parent_name->set_editable(true);
@@ -764,6 +768,13 @@ ScriptCreateDialog::ScriptCreateDialog() {
 	path_error_label = memnew(Label);
 	vb->add_child(path_error_label);
 
+	builtin_warning_label = memnew(Label);
+	builtin_warning_label->set_text(
+			TTR("Note: Built-in scripts have some limitations and can't be edited using an external editor."));
+	vb->add_child(builtin_warning_label);
+	builtin_warning_label->set_autowrap(true);
+	builtin_warning_label->hide();
+
 	status_panel = memnew(PanelContainer);
 	status_panel->set_h_size_flags(Control::SIZE_FILL);
 	status_panel->add_child(vb);
@@ -790,7 +801,7 @@ ScriptCreateDialog::ScriptCreateDialog() {
 	gc->add_child(memnew(Label(TTR("Language:"))));
 	gc->add_child(language_menu);
 
-	default_language = 0;
+	default_language = -1;
 	for (int i = 0; i < ScriptServer::get_language_count(); i++) {
 
 		String lang = ScriptServer::get_language(i)->get_name();
@@ -799,8 +810,9 @@ ScriptCreateDialog::ScriptCreateDialog() {
 			default_language = i;
 		}
 	}
-
-	language_menu->select(default_language);
+	if (default_language >= 0) {
+		language_menu->select(default_language);
+	}
 	current_language = default_language;
 
 	language_menu->connect("item_selected", this, "_lang_changed");
