@@ -1,5 +1,6 @@
 def can_build(env, platform):
-    return platform == 'iphone'
+    return platform == "iphone"
+
 
 def configure(env):
     pass
