@@ -86,7 +86,7 @@ public:
 		ERR_FAIL_COND_MSG(req[0] != "GET" || req[2] != "HTTP/1.1", "Invalid method or HTTP version.");
 
 		String filepath = EditorSettings::get_singleton()->get_cache_dir().plus_file("tmp_js_export");
-		String basereq = "/tmp_js_export";
+		const String basereq = "/tmp_js_export";
 		String ctype = "";
 		if (req[1] == basereq + ".html") {
 			filepath += ".html";
@@ -94,11 +94,19 @@ public:
 		} else if (req[1] == basereq + ".js") {
 			filepath += ".js";
 			ctype = "application/javascript";
+		} else if (req[1] == basereq + ".worker.js") {
+			filepath += ".worker.js";
+			ctype = "application/javascript";
 		} else if (req[1] == basereq + ".pck") {
 			filepath += ".pck";
 			ctype = "application/octet-stream";
-		} else if (req[1] == basereq + ".png") {
-			filepath += ".png";
+		} else if (req[1] == basereq + ".png" || req[1] == "/favicon.png") {
+			// Also allow serving the generated favicon for a smoother loading experience.
+			if (req[1] == "/favicon.png") {
+				filepath = EditorSettings::get_singleton()->get_cache_dir().plus_file("favicon.png");
+			} else {
+				filepath += ".png";
+			}
 			ctype = "image/png";
 		} else if (req[1] == basereq + ".wasm") {
 			filepath += ".wasm";
@@ -244,6 +252,7 @@ void EditorExportPlatformJavaScript::_fix_html(Vector<uint8_t> &p_html, const Re
 
 		String current_line = lines[i];
 		current_line = current_line.replace("$GODOT_BASENAME", p_name);
+		current_line = current_line.replace("$GODOT_PROJECT_NAME", ProjectSettings::get_singleton()->get_setting("application/config/name"));
 		current_line = current_line.replace("$GODOT_HEAD_INCLUDE", p_preset->get("html/head_include"));
 		current_line = current_line.replace("$GODOT_DEBUG_ENABLED", p_debug ? "true" : "false");
 		str_export += current_line + "\n";
@@ -429,6 +438,10 @@ Error EditorExportPlatformJavaScript::export_project(const Ref<EditorExportPrese
 		} else if (file == "godot.js") {
 
 			file = p_path.get_file().get_basename() + ".js";
+		} else if (file == "godot.worker.js") {
+
+			file = p_path.get_file().get_basename() + ".worker.js";
+
 		} else if (file == "godot.wasm") {
 
 			file = p_path.get_file().get_basename() + ".wasm";
@@ -470,11 +483,10 @@ Error EditorExportPlatformJavaScript::export_project(const Ref<EditorExportPrese
 	}
 
 	Ref<Image> splash;
-	String splash_path = GLOBAL_GET("application/boot_splash/image");
-	splash_path = splash_path.strip_edges();
+	const String splash_path = String(GLOBAL_GET("application/boot_splash/image")).strip_edges();
 	if (!splash_path.empty()) {
 		splash.instance();
-		Error err = splash->load(splash_path);
+		const Error err = splash->load(splash_path);
 		if (err) {
 			EditorNode::get_singleton()->show_warning(TTR("Could not read boot splash image file:") + "\n" + splash_path + "\n" + TTR("Using default boot splash image."));
 			splash.unref();
@@ -483,11 +495,32 @@ Error EditorExportPlatformJavaScript::export_project(const Ref<EditorExportPrese
 	if (splash.is_null()) {
 		splash = Ref<Image>(memnew(Image(boot_splash_png)));
 	}
-	String png_path = p_path.get_base_dir().plus_file(p_path.get_file().get_basename() + ".png");
-	if (splash->save_png(png_path) != OK) {
-		EditorNode::get_singleton()->show_warning(TTR("Could not write file:") + "\n" + png_path);
+	const String splash_png_path = p_path.get_base_dir().plus_file(p_path.get_file().get_basename() + ".png");
+	if (splash->save_png(splash_png_path) != OK) {
+		EditorNode::get_singleton()->show_warning(TTR("Could not write file:") + "\n" + splash_png_path);
 		return ERR_FILE_CANT_WRITE;
 	}
+
+	// Save a favicon that can be accessed without waiting for the project to finish loading.
+	// This way, the favicon can be displayed immediately when loading the page.
+	Ref<Image> favicon;
+	const String favicon_path = String(GLOBAL_GET("application/config/icon")).strip_edges();
+	if (!favicon_path.empty()) {
+		favicon.instance();
+		const Error err = favicon->load(favicon_path);
+		if (err) {
+			favicon.unref();
+		}
+	}
+
+	if (favicon.is_valid()) {
+		const String favicon_png_path = p_path.get_base_dir().plus_file("favicon.png");
+		if (favicon->save_png(favicon_png_path) != OK) {
+			EditorNode::get_singleton()->show_warning(TTR("Could not write file:") + "\n" + favicon_png_path);
+			return ERR_FILE_CANT_WRITE;
+		}
+	}
+
 	return OK;
 }
 
@@ -536,23 +569,24 @@ Error EditorExportPlatformJavaScript::run(const Ref<EditorExportPreset> &p_prese
 		return OK;
 	}
 
-	String basepath = EditorSettings::get_singleton()->get_cache_dir().plus_file("tmp_js_export");
-	String path = basepath + ".html";
-	Error err = export_project(p_preset, true, path, p_debug_flags);
+	const String basepath = EditorSettings::get_singleton()->get_cache_dir().plus_file("tmp_js_export");
+	Error err = export_project(p_preset, true, basepath + ".html", p_debug_flags);
 	if (err != OK) {
 		// Export generates several files, clean them up on failure.
 		DirAccess::remove_file_or_error(basepath + ".html");
 		DirAccess::remove_file_or_error(basepath + ".js");
+		DirAccess::remove_file_or_error(basepath + ".worker.js");
 		DirAccess::remove_file_or_error(basepath + ".pck");
 		DirAccess::remove_file_or_error(basepath + ".png");
 		DirAccess::remove_file_or_error(basepath + ".wasm");
+		DirAccess::remove_file_or_error(EditorSettings::get_singleton()->get_cache_dir().plus_file("favicon.png"));
 		return err;
 	}
 
-	IP_Address bind_ip;
-	uint16_t bind_port = EDITOR_GET("export/web/http_port");
+	const uint16_t bind_port = EDITOR_GET("export/web/http_port");
 	// Resolve host if needed.
-	String bind_host = EDITOR_GET("export/web/http_host");
+	const String bind_host = EDITOR_GET("export/web/http_host");
+	IP_Address bind_ip;
 	if (bind_host.is_valid_ip_address()) {
 		bind_ip = bind_host;
 	} else {
