@@ -747,22 +747,7 @@ class EditorExportPlatformUWP : public EditorExportPlatform {
 
 		// TODO: Add resource creation or image rescaling to enable other scales:
 		// 1.25, 1.5, 2.0
-		real_t scales[] = { 1.0 };
-		bool valid_w = false;
-		bool valid_h = false;
-
-		for (int i = 0; i < 1; i++) {
-
-			int w = ceil(p_width * scales[i]);
-			int h = ceil(p_height * scales[i]);
-
-			if (w == p_image->get_width())
-				valid_w = true;
-			if (h == p_image->get_height())
-				valid_h = true;
-		}
-
-		return valid_w && valid_h;
+		return p_width == p_image->get_width() && p_height == p_image->get_height();
 	}
 
 	Vector<uint8_t> _fix_manifest(const Ref<EditorExportPreset> &p_preset, const Vector<uint8_t> &p_template, bool p_give_internet) const {
@@ -992,7 +977,7 @@ class EditorExportPlatformUWP : public EditorExportPlatform {
 
 public:
 	virtual String get_name() const {
-		return "Windows Universal";
+		return "UWP";
 	}
 	virtual String get_os_name() const {
 		return "UWP";
@@ -1176,7 +1161,7 @@ public:
 			err += TTR("Invalid square 71x71 logo image dimensions (should be 71x71).") + "\n";
 		}
 
-		if (!p_preset->get("images/square150x150_logo").is_zero() && !_valid_image((Object::cast_to<StreamTexture>((Object *)p_preset->get("images/square150x150_logo"))), 150, 0)) {
+		if (!p_preset->get("images/square150x150_logo").is_zero() && !_valid_image((Object::cast_to<StreamTexture>((Object *)p_preset->get("images/square150x150_logo"))), 150, 150)) {
 			valid = false;
 			err += TTR("Invalid square 150x150 logo image dimensions (should be 150x150).") + "\n";
 		}
@@ -1204,7 +1189,7 @@ public:
 
 		String src_appx;
 
-		EditorProgress ep("export", "Exporting for Windows Universal", 7, true);
+		EditorProgress ep("export", "Exporting for UWP", 7, true);
 
 		if (p_debug)
 			src_appx = p_preset->get("custom_template/debug");
