@@ -40,6 +40,7 @@ void SkinReference::_skin_changed() {
 	if (skeleton_node) {
 		skeleton_node->_make_dirty();
 	}
+	skeleton_version = 0;
 }
 
 void SkinReference::_bind_methods() {
@@ -321,10 +322,49 @@ void Skeleton::_notification(int p_what) {
 				if (E->get()->bind_count != bind_count) {
 					VS::get_singleton()->skeleton_allocate(skeleton, bind_count);
 					E->get()->bind_count = bind_count;
+					E->get()->skin_bone_indices.resize(bind_count);
+					E->get()->skin_bone_indices_ptrs = E->get()->skin_bone_indices.ptrw();
+				}
+
+				if (E->get()->skeleton_version != version) {
+
+					for (uint32_t i = 0; i < bind_count; i++) {
+						StringName bind_name = skin->get_bind_name(i);
+
+						if (bind_name != StringName()) {
+							//bind name used, use this
+							bool found = false;
+							for (int j = 0; j < len; j++) {
+								if (bonesptr[j].name == bind_name) {
+									E->get()->skin_bone_indices_ptrs[i] = j;
+									found = true;
+									break;
+								}
+							}
+
+							if (!found) {
+								ERR_PRINT("Skin bind #" + itos(i) + " contains named bind '" + String(bind_name) + "' but Skeleton has no bone by that name.");
+								E->get()->skin_bone_indices_ptrs[i] = 0;
+							}
+						} else if (skin->get_bind_bone(i) >= 0) {
+							int bind_index = skin->get_bind_bone(i);
+							if (bind_index >= len) {
+								ERR_PRINT("Skin bind #" + itos(i) + " contains bone index bind: " + itos(bind_index) + " , which is greater than the skeleton bone count: " + itos(len) + ".");
+								E->get()->skin_bone_indices_ptrs[i] = 0;
+							} else {
+								E->get()->skin_bone_indices_ptrs[i] = bind_index;
+							}
+						} else {
+							ERR_PRINT("Skin bind #" + itos(i) + " does not contain a name nor a bone index.");
+							E->get()->skin_bone_indices_ptrs[i] = 0;
+						}
+					}
+
+					E->get()->skeleton_version = version;
 				}
 
 				for (uint32_t i = 0; i < bind_count; i++) {
-					uint32_t bone_index = skin->get_bind_bone(i);
+					uint32_t bone_index = E->get()->skin_bone_indices_ptrs[i];
 					ERR_CONTINUE(bone_index >= (uint32_t)len);
 					vs->skeleton_bone_set_transform(skeleton, i, bonesptr[bone_index].pose_global * skin->get_bind_pose(i));
 				}
@@ -335,6 +375,13 @@ void Skeleton::_notification(int p_what) {
 	}
 }
 
+void Skeleton::clear_bones_global_pose_override() {
+	for (int i = 0; i < bones.size(); i += 1) {
+		bones.write[i].global_pose_override_amount = 0;
+	}
+	_make_dirty();
+}
+
 void Skeleton::set_bone_global_pose_override(int p_bone, const Transform &p_pose, float p_amount, bool p_persistent) {
 
 	ERR_FAIL_INDEX(p_bone, bones.size());
@@ -366,6 +413,7 @@ void Skeleton::add_bone(const String &p_name) {
 	b.name = p_name;
 	bones.push_back(b);
 	process_order_dirty = true;
+	version++;
 	_make_dirty();
 	update_gizmo();
 }
@@ -517,7 +565,7 @@ void Skeleton::clear_bones() {
 
 	bones.clear();
 	process_order_dirty = true;
-
+	version++;
 	_make_dirty();
 }
 
@@ -828,6 +876,7 @@ void Skeleton::_bind_methods() {
 	ClassDB::bind_method(D_METHOD("get_bone_pose", "bone_idx"), &Skeleton::get_bone_pose);
 	ClassDB::bind_method(D_METHOD("set_bone_pose", "bone_idx", "pose"), &Skeleton::set_bone_pose);
 
+	ClassDB::bind_method(D_METHOD("clear_bones_global_pose_override"), &Skeleton::clear_bones_global_pose_override);
 	ClassDB::bind_method(D_METHOD("set_bone_global_pose_override", "bone_idx", "pose", "amount", "persistent"), &Skeleton::set_bone_global_pose_override, DEFVAL(false));
 	ClassDB::bind_method(D_METHOD("get_bone_global_pose", "bone_idx"), &Skeleton::get_bone_global_pose);
 
@@ -849,6 +898,7 @@ void Skeleton::_bind_methods() {
 Skeleton::Skeleton() {
 
 	dirty = false;
+	version = 1;
 	process_order_dirty = true;
 }
 
