@@ -366,7 +366,7 @@ VehicleWheel::VehicleWheel() {
 	engine_traction = false;
 
 	m_steering = real_t(0.);
-	//m_engineForce = real_t(0.);
+	m_engineForce = real_t(0.);
 	m_rotation = real_t(0.);
 	m_deltaRotation = real_t(0.);
 	m_brake = real_t(0.);
