@@ -382,8 +382,14 @@ void ScriptDebuggerRemote::_get_output() {
 		packet_peer_stream->put_var(output_strings.size());
 
 		while (output_strings.size()) {
+			const OutputString &output_string = output_strings.front()->get();
+
+			Array msg_data;
+			msg_data.push_back(output_string.message);
+			msg_data.push_back(output_string.type);
+
+			packet_peer_stream->put_var(msg_data);
 
-			packet_peer_stream->put_var(output_strings.front()->get());
 			output_strings.pop_front();
 		}
 		locking = false;
@@ -680,11 +686,6 @@ void ScriptDebuggerRemote::_send_object_id(ObjectID p_id) {
 		const PropertyInfo &pi = properties[i].first;
 		Variant &var = properties[i].second;
 
-		WeakRef *ref = Object::cast_to<WeakRef>(var);
-		if (ref) {
-			var = ref->get_ref();
-		}
-
 		RES res = var;
 
 		Array prop;
@@ -913,7 +914,7 @@ void ScriptDebuggerRemote::_send_profiling_data(bool p_for_frame) {
 		packet_peer_stream->put_var(8 + to_send * 4);
 	}
 
-	packet_peer_stream->put_var(Engine::get_singleton()->get_frames_drawn()); //total frame time
+	packet_peer_stream->put_var(Engine::get_singleton()->get_idle_frames()); //total frame time
 	packet_peer_stream->put_var(frame_time); //total frame time
 	packet_peer_stream->put_var(idle_time); //idle frame time
 	packet_peer_stream->put_var(physics_time); //fixed frame time
@@ -1146,7 +1147,7 @@ void ScriptDebuggerRemote::_print_handler(void *p_this, const String &p_string,
 	String s = p_string;
 	int allowed_chars = MIN(MAX(sdr->max_cps - sdr->char_count, 0), s.length());
 
-	if (allowed_chars == 0)
+	if (allowed_chars == 0 && s.length() > 0)
 		return;
 
 	if (allowed_chars < s.length()) {
@@ -1162,10 +1163,15 @@ void ScriptDebuggerRemote::_print_handler(void *p_this, const String &p_string,
 		if (overflowed)
 			s += "[...]";
 
-		sdr->output_strings.push_back(s);
+		OutputString output_string;
+		output_string.message = s;
+		output_string.type = p_error ? MESSAGE_TYPE_ERROR : MESSAGE_TYPE_LOG;
+		sdr->output_strings.push_back(output_string);
 
 		if (overflowed) {
-			sdr->output_strings.push_back("[output overflow, print less text!]");
+			output_string.message = "[output overflow, print less text!]";
+			output_string.type = MESSAGE_TYPE_ERROR;
+			sdr->output_strings.push_back(output_string);
 		}
 	}
 	sdr->mutex->unlock();
