@@ -70,7 +70,7 @@ void ColorPicker::_notification(int p_what) {
 		case NOTIFICATION_PARENTED: {
 
 			for (int i = 0; i < 4; i++)
-				set_margin((Margin)i, get_constant("margin"));
+				set_margin((Margin)i, get_margin((Margin)i) + get_constant("margin"));
 		} break;
 		case NOTIFICATION_VISIBILITY_CHANGED: {
 
@@ -624,26 +624,33 @@ void ColorPicker::_screen_pick_pressed() {
 }
 
 void ColorPicker::_focus_enter() {
-	if (c_text->has_focus()) {
+	bool has_ctext_focus = c_text->has_focus();
+	if (has_ctext_focus) {
 		c_text->select_all();
-		return;
+	} else {
+		c_text->select(0, 0);
 	}
+
 	for (int i = 0; i < 4; i++) {
-		if (values[i]->get_line_edit()->has_focus()) {
+		if (values[i]->get_line_edit()->has_focus() && !has_ctext_focus) {
 			values[i]->get_line_edit()->select_all();
-			break;
+		} else {
+			values[i]->get_line_edit()->select(0, 0);
 		}
 	}
 }
 
 void ColorPicker::_focus_exit() {
 	for (int i = 0; i < 4; i++) {
-		values[i]->get_line_edit()->select(0, 0);
+		if (!values[i]->get_line_edit()->get_menu()->is_visible())
+			values[i]->get_line_edit()->select(0, 0);
 	}
 	c_text->select(0, 0);
 }
 
 void ColorPicker::_html_focus_exit() {
+	if (c_text->get_menu()->is_visible())
+		return;
 	_html_entered(c_text->get_text());
 	_focus_exit();
 }
@@ -667,6 +674,7 @@ void ColorPicker::set_presets_visible(bool p_visible) {
 	presets_visible = p_visible;
 	preset_separator->set_visible(p_visible);
 	preset_container->set_visible(p_visible);
+	preset_container2->set_visible(p_visible);
 }
 
 bool ColorPicker::are_presets_visible() const {
