@@ -1987,15 +1987,20 @@ void AnimatedTexture::_update_proxy() {
 	}
 
 	int iter_max = frame_count;
-	while (iter_max) {
+	while (iter_max && !pause) {
 		float frame_limit = limit + frames[current_frame].delay_sec;
 
 		if (time > frame_limit) {
 			current_frame++;
 			if (current_frame >= frame_count) {
-				current_frame = 0;
+				if (oneshot) {
+					current_frame = frame_count - 1;
+				} else {
+					current_frame = 0;
+				}
 			}
 			time -= frame_limit;
+			_change_notify("current_frame");
 		} else {
 			break;
 		}
@@ -2018,6 +2023,33 @@ int AnimatedTexture::get_frames() const {
 	return frame_count;
 }
 
+void AnimatedTexture::set_current_frame(int p_frame) {
+	ERR_FAIL_COND(p_frame < 0 || p_frame >= frame_count);
+
+	RWLockWrite r(rw_lock);
+
+	current_frame = p_frame;
+}
+int AnimatedTexture::get_current_frame() const {
+	return current_frame;
+}
+
+void AnimatedTexture::set_pause(bool p_pause) {
+	RWLockWrite r(rw_lock);
+	pause = p_pause;
+}
+bool AnimatedTexture::get_pause() const {
+	return pause;
+}
+
+void AnimatedTexture::set_oneshot(bool p_oneshot) {
+	RWLockWrite r(rw_lock);
+	oneshot = p_oneshot;
+}
+bool AnimatedTexture::get_oneshot() const {
+	return oneshot;
+}
+
 void AnimatedTexture::set_frame_texture(int p_frame, const Ref<Texture> &p_texture) {
 
 	ERR_FAIL_COND(p_texture == this);
@@ -2141,6 +2173,15 @@ void AnimatedTexture::_bind_methods() {
 	ClassDB::bind_method(D_METHOD("set_frames", "frames"), &AnimatedTexture::set_frames);
 	ClassDB::bind_method(D_METHOD("get_frames"), &AnimatedTexture::get_frames);
 
+	ClassDB::bind_method(D_METHOD("set_current_frame", "frame"), &AnimatedTexture::set_current_frame);
+	ClassDB::bind_method(D_METHOD("get_current_frame"), &AnimatedTexture::get_current_frame);
+
+	ClassDB::bind_method(D_METHOD("set_pause", "pause"), &AnimatedTexture::set_pause);
+	ClassDB::bind_method(D_METHOD("get_pause"), &AnimatedTexture::get_pause);
+
+	ClassDB::bind_method(D_METHOD("set_oneshot", "oneshot"), &AnimatedTexture::set_oneshot);
+	ClassDB::bind_method(D_METHOD("get_oneshot"), &AnimatedTexture::get_oneshot);
+
 	ClassDB::bind_method(D_METHOD("set_fps", "fps"), &AnimatedTexture::set_fps);
 	ClassDB::bind_method(D_METHOD("get_fps"), &AnimatedTexture::get_fps);
 
@@ -2153,6 +2194,9 @@ void AnimatedTexture::_bind_methods() {
 	ClassDB::bind_method(D_METHOD("_update_proxy"), &AnimatedTexture::_update_proxy);
 
 	ADD_PROPERTY(PropertyInfo(Variant::INT, "frames", PROPERTY_HINT_RANGE, "1," + itos(MAX_FRAMES), PROPERTY_USAGE_DEFAULT | PROPERTY_USAGE_UPDATE_ALL_IF_MODIFIED), "set_frames", "get_frames");
+	ADD_PROPERTY(PropertyInfo(Variant::INT, "current_frame", PROPERTY_HINT_NONE, "", 0), "set_current_frame", "get_current_frame");
+	ADD_PROPERTY(PropertyInfo(Variant::BOOL, "pause"), "set_pause", "get_pause");
+	ADD_PROPERTY(PropertyInfo(Variant::BOOL, "oneshot"), "set_oneshot", "get_oneshot");
 	ADD_PROPERTY(PropertyInfo(Variant::REAL, "fps", PROPERTY_HINT_RANGE, "0,1024,0.1"), "set_fps", "get_fps");
 
 	for (int i = 0; i < MAX_FRAMES; i++) {
@@ -2171,6 +2215,8 @@ AnimatedTexture::AnimatedTexture() {
 	fps = 4;
 	prev_ticks = 0;
 	current_frame = 0;
+	pause = false;
+	oneshot = false;
 	VisualServer::get_singleton()->connect("frame_pre_draw", this, "_update_proxy");
 
 #ifndef NO_THREADS
@@ -2621,3 +2667,64 @@ CameraTexture::CameraTexture() {
 CameraTexture::~CameraTexture() {
 	// nothing to do here yet
 }
+
+void ExternalTexture::_bind_methods() {
+	ClassDB::bind_method(D_METHOD("set_size", "size"), &ExternalTexture::set_size);
+	ClassDB::bind_method(D_METHOD("get_external_texture_id"), &ExternalTexture::get_external_texture_id);
+
+	ADD_PROPERTY(PropertyInfo(Variant::VECTOR2, "size"), "set_size", "get_size");
+}
+
+uint32_t ExternalTexture::get_external_texture_id() {
+	return VisualServer::get_singleton()->texture_get_texid(texture);
+}
+
+void ExternalTexture::set_size(const Size2 &p_size) {
+
+	if (p_size.width > 0 && p_size.height > 0) {
+		size = p_size;
+		VisualServer::get_singleton()->texture_set_size_override(texture, size.width, size.height, 0);
+	}
+}
+
+int ExternalTexture::get_width() const {
+	return size.width;
+}
+
+int ExternalTexture::get_height() const {
+	return size.height;
+}
+
+Size2 ExternalTexture::get_size() const {
+	return size;
+}
+
+RID ExternalTexture::get_rid() const {
+	return texture;
+}
+
+bool ExternalTexture::has_alpha() const {
+	return true;
+}
+
+void ExternalTexture::set_flags(uint32_t p_flags) {
+	// not supported
+}
+
+uint32_t ExternalTexture::get_flags() const {
+	// not supported
+	return 0;
+}
+
+ExternalTexture::ExternalTexture() {
+	size = Size2(1.0, 1.0);
+	texture = VisualServer::get_singleton()->texture_create();
+
+	VisualServer::get_singleton()->texture_allocate(texture, size.width, size.height, 0, Image::FORMAT_RGBA8, VS::TEXTURE_TYPE_EXTERNAL, 0);
+	_change_notify();
+	emit_changed();
+}
+
+ExternalTexture::~ExternalTexture() {
+	VisualServer::get_singleton()->free(texture);
+}
