@@ -190,6 +190,10 @@ public:
 	BIND3(shader_set_default_texture_param, RID, const StringName &, RID)
 	BIND2RC(RID, shader_get_default_texture_param, RID, const StringName &)
 
+	BIND2(shader_add_custom_define, RID, const String &)
+	BIND2C(shader_get_custom_defines, RID, Vector<String> *)
+	BIND2(shader_remove_custom_define, RID, const String &)
+
 	/* COMMON MATERIAL API */
 
 	BIND0R(RID, material_create)
@@ -688,6 +692,7 @@ public:
 
 	virtual void set_boot_image(const Ref<Image> &p_image, const Color &p_color, bool p_scale, bool p_use_filter = true);
 	virtual void set_default_clear_color(const Color &p_color);
+	virtual void set_shader_time_scale(float p_scale);
 
 	virtual bool has_feature(Features p_feature) const;
 
